using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace DirectDataTest
{

    public partial class Form1 : Form
    {
        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        public static extern IntPtr FindWindow(string className, string WindowsName);

        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        private extern static Int32 SendMessage(
            IntPtr hwnd,
            int wMsg,
            int wParam,
            IntPtr lParam
            );

        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        private extern static int SendMessage(
            IntPtr hwnd,
            int wMsg,
            IntPtr wParam,
            IntPtr lParam
            );

        [DllImport("Kernel32.dll", CharSet = CharSet.Auto)]
        private extern static IntPtr LocalAlloc(int flag, int val);

        [DllImport("Kernel32.dll", CharSet = CharSet.Auto)]
        private extern static IntPtr LocalFree(IntPtr p);

        private struct COPYDATASTRUCT
        {
            public IntPtr dwData;
            public int cbData;
            public IntPtr lpData;
        };

        private const int WM_COPYDATA = 0x004A;
        private const int WM_SM_REGISTER = 0x8100;
        private const int WM_SM_UNREGISTER = 0x8101;
        private const int ERROR_SUCCESS = 0;
        private const int WM_SM_STARTSTOP = 0x8102;
        private const int WM_SM_GETSTATE = 0x8105;
        private const int SM_RESULT_STARTED = 256;
        private const int SM_RESULT_STOPPED = 512;
        private const int LPTR = 0x0040;

        private const int ST_STOPPED = 1;          // scanner stopped
        private const int ST_STOP = 2;             // scanner stopping in process
        private const int ST_PAUSED = 3;           // scanner paused by AWS
        private const int ST_CONNECTING = 4;
        private const int ST_CONNECTED = 5;         //scanner has established connection
        private const String SM_TITLE = "SerialMagic Professional - Direct Data Mode";

        private bool bRegistered;
        public Form1()
        {
            InitializeComponent();
        }

        private void btnRegister_Click(object sender, EventArgs e)
        {
            logListBox.Items.Add("Searching for SerialMagic window:");

            IntPtr foundWindow = IntPtr.Zero;
            foundWindow = FindWindow(null, SM_TITLE);

            if (foundWindow != IntPtr.Zero)
            {
                logListBox.Items.Add(" - SerialMagic was found.");
                logListBox.Items.Add("Registering to SerialMagic:");

                int rslt = -1;
                rslt = SendMessage(foundWindow, WM_SM_REGISTER, 0, this.Handle);

                if (rslt == ERROR_SUCCESS)
                {
                    logListBox.Items.Add(" - success.");
                    bRegistered = true;
                }
                else
                {
                    logListBox.Items.Add(" - fail.");
                }
            }
            else
            {
                logListBox.Items.Add(" - SerialMagic wasn't found.");
                logListBox.Items.Add(" - Enable Post Direct Data setting in SerialMagic");
            }
            logListBox.SelectedIndex = logListBox.Items.Count - 1;

        }

        private void btnUnregister_Click(object sender, EventArgs e)
        {
            IntPtr foundWindow = IntPtr.Zero;
            foundWindow = FindWindow(null, SM_TITLE);

            if (foundWindow != IntPtr.Zero)
            {
                logListBox.Items.Add("Unregistering to SerialMagic:");

                int rslt = -1;
                rslt = SendMessage(foundWindow, WM_SM_UNREGISTER, 0, this.Handle);

                if (rslt == ERROR_SUCCESS)
                {
                    logListBox.Items.Add(" - success.");
                    bRegistered = false;
                }
                else
                {
                    logListBox.Items.Add(" - fail.");
                }
            }
            else
            {
                logListBox.Items.Add(" - SerialMagic wasn't found.");
            }
            logListBox.SelectedIndex = logListBox.Items.Count - 1;

        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            IntPtr foundWindow = IntPtr.Zero;
            foundWindow = FindWindow(null, SM_TITLE);

            if (foundWindow != IntPtr.Zero)
            {
                int res = SendMessage(foundWindow, WM_SM_STARTSTOP, 0, this.Handle);
                switch (res)
                {
                    case SM_RESULT_STARTED:
                        logListBox.Items.Add("  - SM started");
                        btnStart.Text = "STOP";
                        break;
                    case SM_RESULT_STOPPED:
                        logListBox.Items.Add("  - SM stopped");
                        btnStart.Text = "START";

                        break;
                    default:
                        logListBox.Items.Add("  - failed");
                        break;
                }
                logListBox.SelectedIndex = logListBox.Items.Count - 1;

                if (GetAppState())
                {
                    btnStart.Text = "STOP";
                }
                else
                {
                    btnStart.Text = "START";
                }
            }
            else
            {
                logListBox.SelectedIndex = logListBox.Items.Add("SerialMagic wasn't found.");
            }

        }

        private bool GetAppState()
        {
            IntPtr foundWindow = IntPtr.Zero;
            foundWindow = FindWindow(null, SM_TITLE);

            if (foundWindow != IntPtr.Zero)
            {
                int res = SendMessage(foundWindow, WM_SM_GETSTATE, 0, this.Handle);
                String strText = "SerialMagic state: " + res.ToString();
                logListBox.Items.Add(strText);
                switch (res)
                {
                    case ST_STOPPED:
                    case ST_STOP:
                        return false;
                    case ST_PAUSED:
                    case ST_CONNECTING:
                    case ST_CONNECTED:
                        return true;
                }
            }
            return false;
        }

        private void btnSend_Click(object sender, EventArgs e)
        {
            if (!bRegistered)
            {
                logListBox.SelectedIndex = logListBox.Items.Add("Application wasn't registered to SerialMagic");
                return;
            }

            if (dataTextBox.Text == "")
            {
                logListBox.SelectedIndex = logListBox.Items.Add("Nothing to send to SerialMagic");
                return;
            }

            IntPtr foundWindow = IntPtr.Zero;
            foundWindow = FindWindow(null, SM_TITLE);

            if (foundWindow == IntPtr.Zero)
            {
                logListBox.SelectedIndex = logListBox.Items.Add("SerialMagic wasn't found:");
                return;
            }

            logListBox.SelectedIndex = logListBox.Items.Add("Send data to SerialMagic:");

            IntPtr lParam = IntPtr.Zero;
            IntPtr bytesPtr = IntPtr.Zero;

            if (HexCheckBox.Checked)
            {
                String str = dataTextBox.Text;
                String[] strs = str.Split(' ');
                Byte[] bytes = new Byte[strs.Length];

                for (int i = 0; i < bytes.Length; i++)
                {
                    try
                    {
                        bytes[i] = Convert.ToByte(strs[i], 16);
                    }
                    catch (Exception ex)
                    {
                        logListBox.Items.Add("Invalid input format:"+ex.ToString());
                        return;
                    }
                }

                bytesPtr = LocalAlloc(LPTR, bytes.Length + 1);

                Marshal.Copy(bytes, 0, bytesPtr, bytes.Length);

                COPYDATASTRUCT cbs = new COPYDATASTRUCT();
                cbs.dwData = IntPtr.Zero;
                cbs.cbData = bytes.Length;
                cbs.lpData = bytesPtr;

                lParam = LocalAlloc(LPTR, Marshal.SizeOf(cbs));
                System.Runtime.InteropServices.Marshal.StructureToPtr(cbs, lParam, false);
            }
            else
            {
                byte[] bytes = System.Text.Encoding.ASCII.GetBytes(dataTextBox.Text);

                bytesPtr = LocalAlloc(LPTR, bytes.Length + 1);

                Marshal.Copy(bytes, 0, bytesPtr, bytes.Length);

                COPYDATASTRUCT cbs = new COPYDATASTRUCT();
                cbs.dwData = IntPtr.Zero;
                cbs.cbData = dataTextBox.Text.Length;
                cbs.lpData = bytesPtr;

                lParam = LocalAlloc(LPTR, Marshal.SizeOf(cbs));
                System.Runtime.InteropServices.Marshal.StructureToPtr(cbs, lParam, false);
            }

            int res = SendMessage(foundWindow, WM_COPYDATA, this.Handle, lParam);
            if (res == 1)
            {
                logListBox.SelectedIndex = logListBox.Items.Add("- success.");
            }
            else
            {
                logListBox.SelectedIndex = logListBox.Items.Add("- fail.");
            }

            LocalFree(bytesPtr);
            LocalFree(lParam);

        }
        public void CopyData(ref Message msg)
        {
            COPYDATASTRUCT cds = new COPYDATASTRUCT();
            cds = (COPYDATASTRUCT)System.Runtime.InteropServices.Marshal.PtrToStructure(msg.LParam, typeof(COPYDATASTRUCT));

            if (cds.cbData > 0)
            {
                String tmpString = "Have incoming data:" + cds.cbData.ToString();
                logListBox.Items.Add(tmpString);

                byte[] data = new byte[cds.cbData];
                Marshal.Copy(cds.lpData, data, 0, cds.cbData);
                tmpString = System.Text.Encoding.ASCII.GetString(data, 0, cds.cbData);

                logListBox.SelectedIndex = logListBox.Items.Add(tmpString);
            }
        }

        protected override void WndProc(ref Message msg)
        {
            if(msg.Msg == WM_COPYDATA) {
                CopyData(ref msg);
            }
            base.WndProc(ref msg);
        }
    }
}